'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _cliEngineCommand = require('cli-engine-command');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _plugins = require('./plugins');

var _plugins2 = _interopRequireDefault(_plugins);

var _yarn = require('./yarn');

var _yarn2 = _interopRequireDefault(_yarn);

var _klawSync = require('klaw-sync');

var _klawSync2 = _interopRequireDefault(_klawSync);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class LinkedPlugins extends _cliEngineCommand.Base {
  constructor(config, plugins) {
    super(config);
    this.yarn = new _yarn2.default(config);
    this.plugins = plugins;
    try {
      this._data = this.fs.readJSONSync(this.file);
    } catch (err) {
      if (err.code !== 'ENOENT') throw err;
      this._data = {
        version: '1',
        plugins: []
      };
    }
  }

  /**
   * adds a linked plugin
   * @param {string} p - path of plugin
   */
  async add(p) {
    if (!this.config.debug) this.action.start(`Running prepare script for ${p}`);
    // flow$ignore
    let pjson = require(_path2.default.join(p, 'package.json'));
    await this.prepare(p);
    const name = pjson.name;
    if (this.plugins.plugins.find(p => p.type === 'user' && p.name === name)) {
      throw new Error(`${name} is already installed.
Uninstall with ${this.color.cmd(this.config.bin + ' plugins:uninstall ' + name)}`);
    }
    if (this._data.plugins.includes(p)) throw new Error(`${p} is already linked`);
    // flow$ignore
    let m = require(p);
    if (!m.commands) throw new Error(`${p} does not appear to be a CLI plugin`);
    this._data.plugins.push(p);
    this._save();
    this.action.stop();
  }

  /**
   * removes a linked plugin
   * @param {string} p - path of plugin
   */
  remove(p) {
    this._data.plugins = this._data.plugins.filter(q => q !== p);
    this._save();
  }

  /**
   * list linked plugins
   * @returns {Plugin[]}
   */
  list() {
    return this._data.plugins.map(p => new _plugins.Plugin('link', p, this.config, this.plugins.cache));
  }

  /**
   * runs prepare() on all linked plugins
   */
  async refresh() {
    for (let plugin of this._data.plugins) {
      try {
        await this.prepare(plugin);
      } catch (err) {
        this.warn(`Error refreshing ${plugin}`);
        this.warn(err);
      }
    }
  }

  /**
   * installs plugin dependencies and runs npm prepare if needed
   */
  async prepare(p) {
    let pjson = this._pjson(p);
    await this._install(p);
    if (!pjson.main) throw new Error(`No main script specified in ${_path2.default.join(p, 'package.json')}`);
    let main = _path2.default.join(p, pjson.main);
    if (!this._needsPrepare(p, main)) return;
    this.plugins.clearCache(pjson.name);
    if (!pjson.scripts || !pjson.scripts.prepare) return;
    if (!this.config.debug) this.action.start(`Running prepare script for ${p}`);
    await this.yarn.exec(['run', 'prepare'], { cwd: p });
    this.fs.utimesSync(main, new Date(), new Date());
    this.action.stop();
  }

  _save() {
    this.fs.writeJSONSync(this.file, this._data);
  }

  _needsInstall(p) {
    let modules = _path2.default.join(p, 'node_modules');
    if (!this.fs.existsSync(modules)) return true;
    let modulesInfo = this.fs.statSync(modules);
    let pjsonInfo = this.fs.statSync(_path2.default.join(p, 'package.json'));
    return modulesInfo.mtime < pjsonInfo.mtime;
  }

  _needsPrepare(p, main) {
    if (!this.fs.existsSync(main)) return true;
    let mainInfo = this.fs.statSync(main);
    let modulesInfo = this.fs.statSync(_path2.default.join(p, 'node_modules'));
    if (mainInfo.mtime < modulesInfo.mtime) return true;
    return !!(0, _klawSync2.default)(p, { nodir: true, ignore: '{node_modules,.git}' }).filter(f => f.path.endsWith('.js')).find(f => f.stats.mtime > mainInfo.mtime);
  }

  async _install(p) {
    if (!this._needsInstall(p)) return;
    if (!this.config.debug) this.action.start(`Installing dependencies for ${p}`);
    await this.yarn.exec([], { cwd: p });
    this.fs.utimesSync(_path2.default.join(p, 'node_modules'), new Date(), new Date());
    this.plugins.clearCache(this._pjson(p).name);
    this.action.stop();
  }

  // flow$ignore
  _pjson(p) {
    return require(_path2.default.join(p, 'package.json'));
  }

  get file() {
    return _path2.default.join(this.config.dirs.data, 'linked_plugins.json');
  }
}
exports.default = LinkedPlugins;