'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _cliEngineCommand = require('cli-engine-command');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _rwlockfile = require('rwlockfile');

var _rwlockfile2 = _interopRequireDefault(_rwlockfile);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = class extends _cliEngineCommand.Base {
  get autoupdatefile() {
    return _path2.default.join(this.config.dirs.cache, 'autoupdate');
  }
  get autoupdatelogfile() {
    return _path2.default.join(this.config.dirs.cache, 'autoupdate.log');
  }
  get updatelockfile() {
    return _path2.default.join(this.config.dirs.cache, 'update.lock');
  }
  get binPath() {
    return process.env.CLI_BINPATH;
  }

  async fetchManifest(channel) {
    if (!this.config.s3.host) throw new Error('S3 host not defined');
    try {
      let url = `https://${this.config.s3.host}/${this.config.name}/channels/${channel}/${process.platform}-${process.arch}`;
      let manifest = await this.http.get(url);
      return manifest;
    } catch (err) {
      if (err.statusCode === 403) throw new Error(`HTTP 403: Invalid channel ${channel}`);
      throw err;
    }
  }

  async update(manifest) {
    if (!this.config.s3.host) throw new Error('S3 host not defined');
    // TODO: read sha256
    let url = `https://${this.config.s3.host}/${this.config.name}/channels/${manifest.channel}/${this.base(manifest)}.tar.gz`;
    let stream = await this.http.stream(url);
    let dir = _path2.default.join(this.config.dirs.data, 'cli');
    let tmp = _path2.default.join(this.config.dirs.data, 'cli_tmp');
    await this.extract(stream, tmp);
    let unlock = await _rwlockfile2.default.write(this.updatelockfile, { skipOwnPid: true });
    this.fs.removeSync(dir);
    this.fs.renameSync(_path2.default.join(tmp, this.base(manifest)), dir);
    this.fs.removeSync(tmp);
    unlock();
  }

  extract(stream, dir) {
    const zlib = require('zlib');
    const tar = require('tar-stream');

    return new Promise(resolve => {
      this.fs.removeSync(dir);
      let extract = tar.extract();
      extract.on('entry', (header, stream, next) => {
        let p = _path2.default.join(dir, header.name);
        let opts = { mode: header.mode };
        switch (header.type) {
          case 'directory':
            this.fs.mkdirpSync(p, opts);
            next();
            break;
          case 'file':
            stream.pipe(this.fs.createWriteStream(p, opts));
            break;
          case 'symlink':
            // ignore symlinks since they will not work on windows
            next();
            break;
          default:
            throw new Error(header.type);
        }
        stream.resume();
        stream.on('end', next);
      });
      extract.on('finish', resolve);
      stream.pipe(zlib.createGunzip()).pipe(extract);
    });
  }

  base(manifest) {
    return `${this.config.name}-v${manifest.version}-${process.platform}-${process.arch}`;
  }

  async restartCLI() {
    await _rwlockfile2.default.read(this.updatelockfile);
    _rwlockfile2.default.unreadSync(this.updatelockfile);
    const { spawnSync } = require('child_process');
    if (!this.binPath) return;
    const { status } = spawnSync(this.binPath, process.argv.slice(2), { stdio: 'inherit', shell: true });
    this.exit(status);
  }

  get autoupdateNeeded() {
    try {
      const moment = require('moment');
      const stat = this.fs.statSync(this.autoupdatefile);
      return moment(stat.mtime).isBefore(moment().subtract(4, 'hours'));
    } catch (err) {
      if (err.code !== 'ENOENT') console.error(err.stack);
      return true;
    }
  }

  async autoupdate() {
    try {
      if (!this.autoupdateNeeded) return;
      this.fs.writeFileSync(this.autoupdatefile, '');
      if (this.config.updateDisabled) await this.warnIfUpdateAvailable();
      await this.checkIfUpdating();
      let fd = this.fs.openSync(this.autoupdatelogfile, 'a');
      if (!this.binPath) return;
      const { spawn } = require('child_process');
      spawn(this.binPath, ['update'], { stdio: [null, fd, fd] }).on('error', e => this.warn(e, 'autoupdate:'));
    } catch (e) {
      this.warn(e, 'autoupdate:');
    }
  }

  async warnIfUpdateAvailable() {
    const manifest = await this.fetchManifest(this.config.channel);
    let local = this.config.version.split('.');
    let remote = manifest.version.split('.');
    if (local[0] !== remote[0] || local[1] !== remote[1]) {
      this.warn(`${this.config.name}: update available from ${this.config.version} to ${manifest.version}`);
    }
  }

  async checkIfUpdating() {
    if (await _rwlockfile2.default.hasWriter(this.updatelockfile)) {
      this.warn(`${this.config.name}: warning: update in process`);
      await this.restartCLI();
    } else await _rwlockfile2.default.read(this.updatelockfile);
  }
};